(function ($) {
  Drupal.behaviors.views_gantt  = {
    attach: function (context) {
      var gantt = new GanttChart();
      gantt.setImagePath(Drupal.settings.basePath + "sites/all/libraries/dhtmlxgantt/codebase/imgs/");
      gantt.setEditable(true);
      gantt.showContextMenu(false);
      gantt.showDescTask(true,'p,n,e,d');

      // Resize tasks
      gantt.attachEvent("onTaskEndResize", function(task) {
        var project_id = Drupal.settings.views_gantt.project_id;
        var project = gantt.getProjectById(project_id);

        jQuery.ajax({
          url: Drupal.settings.basePath + "views_gantt/update/" + task.getId(),
          type: "POST",
          data: {
            fields : {
              duration_field : task.getDuration(),
              project_date_field : Math.round(project.getStartDate().getTime() / 1000) - task.getEST().getTimezoneOffset() * 60
            },
            project_id : project_id,
            view_name : Drupal.settings.views_gantt.view_name
          }
        });
      });

      // Drag tasks
      gantt.attachEvent("onTaskEndDrag", function(task) {
        var project_id = Drupal.settings.views_gantt.project_id;
        var project = gantt.getProjectById(project_id);

        jQuery.ajax({
          url: Drupal.settings.basePath + "views_gantt/update/" + task.getId(),
          type: "POST",
          data: {
            fields : {
              date_field : Math.round(task.getEST().getTime() / 1000) - task.getEST().getTimezoneOffset() * 60,
              project_date_field : Math.round(project.getStartDate().getTime() / 1000) - project.getStartDate().getTimezoneOffset() * 60
            },
            project_id : project_id,
            view_name : Drupal.settings.views_gantt.view_name
          }
        });
      });

      // Create gantt chart
      gantt.create("GanttDiv");
      gantt.loadData(Drupal.settings.basePath + 'views_gantt/project.xml',true,true);

      var project = gantt.getProjectById(Drupal.settings.views_gantt.project_id);

      // Create dialog
      $(".gantt-task-progress").dialog({
        autoOpen: false,
        title: 'Change progress',
        buttons: [ { text: "Ok", click: function() {
          $(this).dialog('close');

          var task_id = $(this).children('[name="task_id"]').val();
          var task_progress = $(this).children('[name="task_progress"]').val();
          progressChange(task_id, task_progress, gantt);
        }} ]
      });

      // Go to node after doubleclick on title
      $('.taskNameItem').dblclick(function() {
        window.open(Drupal.settings.basePath + 'node/' + $(this).attr('id'));
      });

      // Change the progress after doubleclick on task description
      $('.descTask').dblclick(function() {
        var task_id = $(this).prevAll('div:first').attr('id');
        var task = project.getTaskById(task_id);

        $('.gantt-task-progress [name="task_id"]').val(task_id);
        $('.gantt-task-progress [name="task_progress"]').val(task.getPercentCompleted());
        $(".gantt-task-progress").dialog('open');
      });

      var left_menu_offset = 13;
      if (!$('#GanttDiv .taskPanel').height()) {
        var height = 0;
       left_menu_offset = 15;
        $('#GanttDiv .taskPanel > div').each(function() {
          height += $(this).height() + 5;
        });
        $('#GanttDiv .taskPanel').height(height);
        $('#GanttDiv td div > div').first().height(height);
      }  
      $('#GanttDiv td div').first().css('top', left_menu_offset + 'px');    
    }
  }

  function progressChange(task_id, progress, gantt) {
    var project_id = Drupal.settings.views_gantt.project_id;
    var project = gantt.getProjectById(project_id);
    var task = project.getTaskById(task_id);
    task.setPercentCompleted(progress);
    project.setPercentCompleted(project.getPercentCompleted());

    if (progress >= 0 && progress <= 100) {
      jQuery.ajax({
        url: Drupal.settings.basePath + "views_gantt/update/" + task_id,
        type: "POST",
        data: {
          fields : {progress_field : progress},
          project_id : project_id,
          view_name : Drupal.settings.views_gantt.view_name
        }
      });
    }
  }
})(jQuery);
