<?php

/**
 * @file
 * Contains the list style plugin.
 */

/**
 * Style plugin to render Gantt charts.
 *
 * @ingroup views_style_plugins
 */
class views_gantt_plugin_style_gantt extends views_plugin_style_list {
  private $option_fields;
  private $options_node;
  /**
   * Set default options.
   */
  function option_definition() {
    $options = parent::option_definition();

    $options['height'] = array('default' => '');
    $options['class'] = array('default' => '');
    $options['wrapper_class'] = array('default' => 'item-list');
    $options['id_field'] = array('default' => '');
    $options['name_field'] = array('default' => '');
    $options['parent_field'] = array('default' => '');
    $options['date_field'] = array('default' => '');
    $options['duration_field'] = array('default' => '');
    $options['progress_field'] = array('default' => '');
    $options['project_id_field'] = array('default' => '');
    $options['project_node_type'] = array('default' => '');
    $options['project_date_field'] = array('default' => '');
    $options['parent_id_field'] = array('default' => '');
    $options['predecessor_id_field'] = array('default' => '');
    return $options;
  }
  /**
   * Style option form.
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    $fields = array('' => t('<None>'));

    foreach ($this->display->handler->get_handlers('field') as $field => $handler) {
      if ($label = $handler->label()) {
        $fields[$field] = $label;
      }
      else {
        $fields[$field] = $handler->ui_name();
      }
    }

    $form['height'] = array(
      '#type' => 'textfield',
      '#title' => t('Height of Gantt Chart'),
      '#default_value' => $this->options['height'],
      '#description' => t('Height of Gantt Chart (in px).'),
      '#size' => '5',
    );
    $form['id_field'] = array(
      '#type' => 'select',
      '#title' => t('ID field'),
      '#options' => $fields,
      '#default_value' => $this->options['id_field'],
      '#description' => t('Select the field that contains nid of each record.'),
      '#required' => TRUE,
    );
    $form['name_field'] = array(
      '#type' => 'select',
      '#title' => t('Name field'),
      '#options' => $fields,
      '#default_value' => $this->options['name_field'],
      '#description' => t('Select the field that contains name of each record.'),
      '#required' => TRUE,
    );
    $form['date_field'] = array(
      '#type' => 'select',
      '#title' => t('Date field'),
      '#options' => $fields,
      '#default_value' => $this->options['date_field'],
      '#description' => t('Select the field that contains the date (timestamp or valid <a href="http://www.php.net/manual/en/datetime.formats.php" target="_blank">date format</a>) of the node in the selected row.'),
      '#required' => TRUE,
    );
    $form['duration_field'] = array(
      '#type' => 'select',
      '#title' => t('Duration field'),
      '#options' => $fields,
      '#default_value' => $this->options['duration_field'],
      '#description' => t('Select the field that contains the duration (in hours) of the node in the selected row.'),
      '#required' => TRUE,
    );
    $form['progress_field'] = array(
      '#type' => 'select',
      '#title' => t('Progress field'),
      '#options' => $fields,
      '#default_value' => $this->options['progress_field'],
      '#description' => t('Select the field that contains the progress of the node in percents.'),
      '#required' => TRUE,
    );
    $form['project_id_field'] = array(
      '#type' => 'select',
      '#title' => t('Project ID field'),
      '#options' => $fields,
      '#default_value' => $this->options['project_id_field'],
      '#description' => t("Select the field that contains nid of the record's project node."),
      '#required' => TRUE,
    );
    $form['project_node_type'] = array(
      '#type' => 'select',
      '#title' => t('Project node type'),
      '#options' => node_type_get_names(),
      '#default_value' => $this->options['project_node_type'],
      '#description' => t('Select type of project nodes.'),
    );
    $form['project_date_field'] = array(
      '#type' => 'select',
      '#title' => t('Project date field'),
      '#options' => $fields,
      '#default_value' => $this->options['project_date_field'],
      '#description' => t("Select the field that contains the date (timestamp or valid <a href='http://www.php.net/manual/en/datetime.formats.php' target='_blank'>date format</a>) of the record's project node."),
      '#required' => TRUE,
    );
    $form['parent_id_field'] = array(
      '#type' => 'select',
      '#title' => t('Parent ID field'),
      '#options' => $fields,
      '#default_value' => $this->options['parent_id_field'],
      '#description' => t("Select the field that contains nid of the record's parent node."),
    );
    $form['predecessor_id_field'] = array(
      '#type' => 'select',
      '#title' => t('Predecessor ID field'),
      '#options' => $fields,
      '#default_value' => $this->options['predecessor_id_field'],
      '#description' => t("Select the field that contains nid of the record's predecessor node."),
    );
  }
  /**
   * Render the given style.
   */
  function render() {
    // Check for live preview.
    if (isset($this->view->live_preview)) {
      return t('Gantt Chart not compatible with live preview.');
    }

    // Get real names of fields that we put in style settings.
    // If field come from Fields API, we get it's name
    // Otherwise (if field is 'node_title' for example) we get field alias.
    $this->option_fields = array();
    $this->option_fields_node = array();
    $view_fields = $this->view->field;
    foreach ($this->view->display_handler->get_handlers('field') as $key => $handler) {
      if (isset($view_fields[$key]) && isset($handler->field_info)) {
        $this->option_fields[$key] = 'field_' . $key;
      }
      else {
        $this->option_fields[$key] = $handler->field_alias;
      }
    }

    // Some fields in $this->options can have
    // incorrect names (e.g. field_date_1).
    // We need to create new array which contains
    // correct field names to use it in
    // views_gantt_get_node_field_value() function.
    foreach ($this->options as $key => $value) {
      if (is_string($value) && isset($view_fields[$value])) {
        $this->options_node[$key] = $view_fields[$value]->field;
      }
    }

    // Save view name to variable.
    variable_set('views_gantt_view_name', $this->view->name);

    // Get project data.
    $project_id = $this->views_gantt_get_field_value($this->view->result[0], 'project_id_field');
    $node = node_load($project_id);
    $project_date = $this->views_gantt_get_field_value($this->view->result[0], 'project_date_field');

    if (!$project_date) {
      drupal_set_message(t('Gantt Chart requires filled project date.'), 'warning');
      return NULL;
    }

    $project = array(
      'name' => $node->title,
      'id' => $node->nid,
      'startdate' => $project_date,
    );

    // Get array of tasks.
    $tasks = array();
    foreach ($this->view->result as $row) {
      $tasks[$this->views_gantt_get_field_value($row, 'id_field')] = array(
        'name' => $this->views_gantt_get_field_value($row, 'name_field'),
        'est' => $this->views_gantt_get_field_value($row, 'date_field'),
        'duration' => $this->views_gantt_get_field_value($row, 'duration_field'),
        'percentcompleted' => $this->views_gantt_get_field_value($row, 'progress_field'),
        'predecessortasks' => $this->views_gantt_get_field_value($row, 'predecessor_id_field'),
        'parent_id' => $this->views_gantt_get_field_value($row, 'parent_id_field'),
        'childtasks' => array(),
        'child_ids' => array(),
      );
    }

    // Allow to alter tasks array before next modifications.
    drupal_alter('views_gantt_tasks_prerender', $tasks);

    // If we have parent or predecessor tasks that doesn't match the filter,
    // we need to get them to avoid gantt chart errors.
    foreach ($tasks as $task_id => $task) {
      if ($task['predecessortasks'] && !array_key_exists($task['predecessortasks'], $tasks)) {
        $this->views_gantt_get_missing_task($task['predecessortasks'], $tasks, $task_id);
      }
      if ($task['parent_id'] && !array_key_exists($task['parent_id'], $tasks)) {
        $this->views_gantt_get_missing_task($task['parent_id'], $tasks, $task_id);
      }

      // If task has no duration, try to get it's child to calculate duration
      if (!$task['duration']) {
        $child_nids = db_select($view_fields[$this->options['parent_id_field']]->table, 't')
          ->fields('t', array('entity_id'))
          ->condition('entity_type', 'node')
          ->condition($view_fields[$this->options['parent_id_field']]->real_field, $task_id)
          ->execute()
          ->fetchCol();
        foreach ($child_nids as $nid) {
          $this->views_gantt_get_missing_task($nid, $tasks, $task_id);
        }
      }
    }

    foreach ($tasks as $task_id => $task) {
      // Check if there are any incorrect tasks in array and remove them.
      $is_incorrect = ($task['predecessortasks'] && !array_key_exists($task['predecessortasks'], $tasks))
        || ($task['parent_id'] && !array_key_exists($task['parent_id'], $tasks));

      if ($is_incorrect) {
        unset($tasks[$task_id]);
      } else if ($task['parent_id']) {
        $tasks[$task['parent_id']]['child_ids'][] = $task_id;
      }
    }
 
    // If task has no duration, try to calculate duration by summing durations of subtasks.
    // If parent task has lower duration than the sum of all it's child tasks,
    // we increase the duration of a this task to the sum of all it's subtasks.    
    foreach ($tasks as $task_id => &$task) {
      if (!$task['duration']) {
        $this->views_gantt_fix_task_duration($task_id, $tasks);
      } else if (isset($task['child_ids'])) {
        $children_duration = 0;
        foreach ($task['child_ids'] as $value) {
          if (isset($tasks[$value])) {
            $children_duration += $tasks[$value]['duration'];
          }
        }

        if ($children_duration > $task['duration']) {
          $task['duration'] = $children_duration;
        }
      }
    }

    // When we filtering view results, we can encounter with situation, when 
    // there are only predecessor tasks in result set, but not tasks that we searched.
    // In that case we just clear results.
    $clear_result_set = TRUE;
    foreach ($this->view->result as $value) {
      if (isset($tasks[$value->nid])) {
        $clear_result_set = FALSE;
        break;
      }
    }
    if ($clear_result_set) {
      $tasks = array();
    }

    // Save data in session to use it when we will build XML file for chart.
    $_SESSION['views_gantt']['project'] = $project;
    $_SESSION['views_gantt']['tasks'] = $tasks;

    // Add css to fix chart style.
    drupal_add_css(drupal_get_path('module', 'views_gantt') . "/css/reset.css");
    // Add js.
    drupal_add_js(drupal_get_path('module', 'views_gantt') . "/js/views_gantt.js");
    // Add dhtmlxgantt library.
    libraries_load('dhtmlxgantt');
    // Add jquery ui Dialog library.
    drupal_add_library('system', 'ui.dialog');
    
    // Add globals variables to use it in js.
    $settings = array(
      'views_gantt' => array(
        'view_name' => $this->view->name,
        'project_id' => $project_id,
      ),
    );
    drupal_add_js($settings, 'setting');    

    return parent::render();
  }

  /**
   * Returns value of specific field.
   * 
   * @param object $row
   *   Node object from view result array
   * @param string $field_options_name
   *   Option key from views_gantt_plugin_style_gantt::option_definition()
   * 
   * @return string
   *   Field value from node object
   */
  function views_gantt_get_field_value($row, $field_options_name) {
    $field_name = $this->options[$field_options_name];
    $field_value = '';

    // If field value is array, we try to get it's raw value,
    // if it's not possible, we get rendered value.
    if (isset($this->option_fields[$field_name]) && !empty($this->option_fields[$field_name])) {
      $field_value = $row->{$this->option_fields[$field_name]};
      if (is_array($field_value)) {
        if (isset($field_value[0]['raw']['value'])) {
          $field_value = $field_value[0]['raw']['value'];
        }
        elseif (isset($field_value[0]['rendered']['#markup'])) {
          $field_value = $field_value[0]['rendered']['#markup'];
        }
      }

      // If field should provide date, we need to return it's
      // value in specific date format (Y,m,j).
      if (!is_array($field_value) && ($field_options_name == 'date_field'
       || $field_options_name == 'project_date_field')) {
        views_gantt_normalize_date_field($field_value);
      }
    }

    return is_array($field_value) && empty($field_value) ? '' : $field_value;
  }

  /**
   * Get tasks that have been excluded by exposed filters.
   */
  function views_gantt_get_missing_task($task_to_load, &$tasks, $task_id) {
    $fields = $this->options;
    $node = node_load($task_to_load);
    if ($node) {
      $loaded_task = array(
        'name' => $this->views_gantt_get_node_field_value($node, 'name_field'),
        'est' => $this->views_gantt_get_node_field_value($node, 'date_field'),
        'duration' => $this->views_gantt_get_node_field_value($node, 'duration_field'),
        'percentcompleted' => $this->views_gantt_get_node_field_value($node, 'progress_field'),
        'predecessortasks' => $this->views_gantt_get_node_field_value($node, 'predecessor_id_field'),
        'parent_id' => $this->views_gantt_get_node_field_value($node, 'parent_id_field'),
        'project_id' => $this->views_gantt_get_node_field_value($node, 'project_id_field'),
        'childtasks' => '',
      );

      // If parent/predecessor is incorrect (don't have date),
      // we don't include it and also exclude it's child/successor.
      if (!$loaded_task['est']) {
        unset($tasks[$task_id]);
      }
      else {
        $tasks[$task_to_load] = $loaded_task;

        // If received task has parent or predecessor, we need to get them too.
        if ($loaded_task['predecessortasks'] && !array_key_exists($loaded_task['predecessortasks'], $tasks)) {
          $this->views_gantt_get_missing_task($loaded_task['predecessortasks'], $tasks, $task_to_load);
        }
        if ($loaded_task['parent_id'] && !array_key_exists($loaded_task['parent_id'], $tasks)) {
          $this->views_gantt_get_missing_task($loaded_task['parent_id'], $tasks, $task_to_load);
        }

        // If task has no duration, try to get it's child to calculate duration
        $get_child = !$loaded_task['duration'] && isset($this->view->field[$this->options['parent_id_field']]->real_field);
        if ($get_child) {
          $child_nids = db_select($this->view->field[$this->options['parent_id_field']]->table, 't')
            ->fields('t', array('entity_id'))
            ->condition('entity_type', 'node')
            ->condition($this->view->field[$this->options['parent_id_field']]->real_field, $task_to_load)
            ->execute()
            ->fetchCol();
          foreach ($child_nids as $nid) {
            $this->views_gantt_get_missing_task($nid, $tasks, $task_to_load);
          }
        }        
      }
    }
  }

  /**
   * Returns value of specific field from node.
   * 
   * @param object $node
   *   Node object received via node_load()
   * @param string $field_options_name
   *   Option key from views_gantt_plugin_style_gantt::option_definition()
   * 
   * @return string
   *   Field value from node object
   */
  function views_gantt_get_node_field_value($node, $field_options_name) {
    $field_name = $this->options_node[$field_options_name];
    $field_value = '';
    $lang = LANGUAGE_NONE;

    if (isset($node->$field_name) && !empty($node->$field_name)) {
      $field_value = $node->$field_name;
      if (is_array($field_value)) {
        // Get field info.
        $field_info = field_info_field($field_name);

        // If field is translatable, we check if
        // it has an index equals to node language.
        $is_translatable = field_is_translatable('node', $field_info);
        if ($is_translatable && isset($node->{$field_name}[$node->language])) {
          $lang = $node->language;
        }

        $value_keys = array_keys($field_info['columns']);
        $field_value = $node->{$field_name}[$lang][0][$value_keys[0]];
      }

      // If field should provide date, we need to return it's
      // value in specific date format (Y,m,j).
      if (!is_array($field_value) && ($field_options_name == 'date_field'
       || $field_options_name == 'project_date_field')) {
        views_gantt_normalize_date_field($field_value);
      }
    }

    return $field_value;
  }

  /**
   * Calculates task duration by summing durations of child tasks.
   * If duration can't be calculated, this function will delete task from results.
   * 
   * @param int $key
   *  ID of task which duration should be calculated
   * @param array $tasks
   *  Array of tasks
   */
  function views_gantt_fix_task_duration($key, &$tasks) {
    $task = &$tasks[$key];
    $task['duration'] = 0;
    if (isset($task['child_ids'])) {
      foreach ($task['child_ids'] as $children_id) {
        if (isset($tasks[$children_id])) {
          if (!$tasks[$children_id]['duration'] && !empty($tasks[$children_id]['child_ids'])) {
            $this->views_gantt_fix_task_duration($children_id, $tasks);
          }
          if (isset($tasks[$children_id])) {
            $task['duration'] += $tasks[$children_id]['duration'];
          }
        }
      }       
    }

    if (!$task['duration']) {
      unset($tasks[$key]);
    }
  }
}
