<?php
/**
 * Template class for document export plugin
 * 
 */

abstract class document_export {
  
  protected $pages;
  
  protected $header;
  
  protected $footer;
  
  protected $document;
  
  function __construct($options){
    if(isset($options['pages'])){
      $pages_array = $options['pages'];
      if(is_array($pages_array)) $this->pages = $pages_array;
      elseif (is_string($pages_array)) $this->pages = array($pages_array);
      else $this->pages = array();
    }
    if(isset($options['header']))
      $this->header = $options['header'];
    if(isset($options['footer']))
      $this->footer = $options['footer'];
  }
  
  abstract function save($uri, $filename);
  
  abstract function stream($filename);
  
  public function test(){
    return 'Whats up?';
  }
  protected function stream_file($filename, $file, $mimetype){
    header('Content-Description: File Transfer');
    header('Content-Type: ' . $mimetype);
    header('Content-Disposition: attachment; filename=' . $filename);
    header('Content-Transfer-Encoding: binary');
    header('Expires: 0');
    header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
    header('Pragma: public');
    header('Content-Length: ' . filesize($file));
    ob_clean();
    flush();
    $status = readfile($file);
    unlink($file);
    exit;
  }
}

abstract class DocumentExportData{
  
  public $title;
  
  public $body;
  
  public $depth = 0;
  
  public $theme_callback = '';
  
  public $children = array();
  
  function __construct($options = array(), $depth = 0){
    $this->depth = $depth;
    $this->title = (isset($options['title'])) ? $options['title'] : FALSE;
    $this->title = (isset($options['body'])) ? $options['body'] : FALSE;
  }
  
  public function getHtml(){
    theme($this->theme_callback, $this);
  }
  
  public function addChildren($type, $options){
    switch ($type) {
      case 'page':
        $page = new DocumentExportDataPage($options);
        $page->depth = $this->depth + 1;
        $this->children[] = $page;
        return $page;
      
      case 'task':
        $task = new DocumentExportDataTask($options);
        $task->depth = $this->depth + 1;
        $this->children[] = $task;
        return $task;
        
      case 'comment':
        $comment = new DocumentExportDataComment($options);
        $comment->depth = $this->depth + 1;
        $this->children[] = $comment;
        
      default:
        return FALSE;
    }
  }
}

class DocumentExportDataPage extends DocumentExportData{
  
  function __construct($options = array(), $depth = 0){
    parent::__construct($options, $depth);
    $this->nid = (isset($options['nid'])) ? $options['nid'] : FALSE;
    $this->theme_callback = 'erpal_document_export_page';
  }
}

class DocumentExportDataTask extends DocumentExportData{
  
  public $nid;
  
  function __construct($options = array(), $depth = 0){
    parent::__construct($options, $depth);
    $this->nid = (isset($options['nid'])) ? $options['nid'] : FALSE;
    $this->theme_callback = 'erpal_document_export_task';
  }
}

class DocumentExportDataComment extends DocumentExportData{
  
  public $cid;
  
  function __construct($options = array(), $depth){
    parent::__construct($options);
    $this->cid = (isset($options['cid'])) ? $options['cid'] : FALSE; 
    $this->theme_callback = 'erpal_document_export_comment';
  }
}

